*compute occupational segregation indices with bootstrap s.e. and bias correction
*data in memory should be only the obs in the estimation sample; bootstrap weights contain all relevant info about other obs

program computeseg_bs
version 13.1

  syntax, cat(varname) geo(varname) weight(varname) bsweights(varlist) yearlist(numlist) saveresults(string asis) 
  *cat is the name of a variable like occupation or industry
  *we calculate how segregated cat is across states

  local nyears : word count `yearlist'
  local bsamples : word count `bsweights'

  *1. identify cat values present in all years
  qui {
    preserve
    by year `cat', sort: keep if _n==1
    collapse (count) years=year, by(`cat') fast
    keep if years==`nyears'
    tempfile keepcat_y
    save `keepcat_y', replace
    restore
    }

  *2. turn cat values not present in all years into a generic "other" category
  *has to be done in both the collapsed data and the raw data
  qui merge m:1 `cat' using `keepcat_y', assert(match master)
  qui replace `cat'=. if _merge~=3
  drop _merge

  *3. renumber cat and geo
  tempvar catvar geovar
  qui egen `catvar'=group(`cat'), missing
  sum `catvar', meanonly
  local catmax=r(max)
  qui egen `geovar'=group(`geo')
  sum `geovar', meanonly
  local geomax=r(max)

  *4. save the data
  tempfile base_y
  qui save `base_y', replace

  *5. computations
  tempname results
  mat `results'=J(`nyears',`bsamples'+2,.)
  local cnames "year H"
  forvalues i=1/`bsamples' {
    local cnames "`cnames' H_bs`i'"
    }
  mat colnames `results' = `cnames'
  tempname H
  forvalues row=1/`nyears' {
    local y : word `row' of `yearlist'
    di `y'
    use `base_y' if year==`y', clear
    sort `geovar'
    mata: compute1("`catvar'","`geovar'",`catmax',`geomax',"`weight' `bsweights'","`H'")
    mat `results'[`row',1]=`y'
    mat `results'[`row',2]=`H'
    }

  *6. save results
  drop _all
  qui {
    svmat `results', names(col)
    egen H_m_bs=rowmean(H_bs*)
    egen H_se_bs=rowsd(H_bs*)
    gen H_v_bs=H_se_bs*H_se_bs
    gen H_bias_bs=H_m_bs-H
    gen H_bc=H-H_bias_bs
    }
  list year H H_bc H_se_bs, noobs clean
  save `saveresults'

end

mata:
  
  function compute1(string scalar catvar, string scalar geovar, real scalar catmax, real scalar geomax, string scalar wvars, string scalar Hout)
    {
    /* assumes sorted by geo */
    /* assumes catvar is numbered 1 ... catmax, geovar is numbered 1 ... geomax */
    real matrix geo, cat, w, georowids, catrowids, Es, Ns, Nsc, Nc, N, E, H, wvarindex, pi
    real scalar g, c, n_weights, i
    st_view(geo,.,geovar)
    georowids=panelsetup(geo,1)
    wvarindex=st_varindex(tokens(wvars))
    n_weights=cols(wvarindex)
    Es=J(geomax,n_weights,0)
    Ns=J(geomax,n_weights,.)
    Nc=J(catmax,n_weights,0)
    for(g=1;g<=geomax;g++) {
      /* Ns = sum of weight within geo area g */
      st_view(w,georowids[g,.],wvarindex)
      Ns[g,.]=colsum(w)
      /* Es = - sum_c pi_cs * ln(pi_cs) */
      /* have to do inefficiently because some values of catvar might not be observed */
      /* convenient to store sum of weight within g and c */
      st_view(cat,georowids[g,.],catvar)
      for(c=1;c<=catmax;c++) {
        Nsc=colsum(w:*(cat:==c))
        Nc[c,.]=Nc[c,.]+Nsc
        pi=Nsc:/Ns[g,.]
        Es[g,.]=Es[g,.]-pi:*editmissing(ln(pi),0)
        }
      }
    /* N = sum of weight overall */
    N=colsum(Ns)
    /* E = -sum_c pi_c * ln(pi_c) */
    E=J(1,n_weights,0)
    for(c=1;c<=catmax;c++) {
      pi=Nc[c,.]:/N
      E=E-pi:*editmissing(ln(pi),0)
      }
    /* finish */
    H=colsum(Ns:*(E:-Es)):/(E:*N)
    st_matrix(Hout,H)
    }

end


/*

H = sum_s  n_s (E-E_s) / E N

where

E_s = sum_o pi_so ln(1/pi_so)

E = sum_o pi_o ln(1/pi_o)

N = total workers in nation

N_s = total workers in state

*/
